@@ -38,7 +38,7 @@ using Kerfuffle::AddToArchive;
 int main(int argc, char **argv)
 {
     KAboutData aboutData("ark", 0, ki18n("Ark"),
-                         "2.18", ki18n("KDE Archiving tool"),
+                         "2.19", ki18n("KDE Archiving tool"),
                          KAboutData::License_GPL,
                          ki18n("(c) 1997-2011, The Various Ark Developers"),
                          KLocalizedString(),
@@ -86,7 +86,7 @@ int main(int argc, char **argv)
                         ki18n("bkisofs code"),
                         QByteArray(),
                         "http://littlesvr.ca/misc/contactandrew.php");
-    aboutData.setProgramIconName(QLatin1String("ark"));
+    aboutData.setProgramIconName(QLatin1String( "utilities-file-archiver" ));
 
     KCmdLineArgs::init(argc, argv, &aboutData);
     KCmdLineOptions option;
