@@ -2,7 +2,7 @@
  * ark -- archiver for the KDE project
  *
  * Copyright (C) 2009 Harald Hvaal <haraldhv@stud.ntnu.no>
- * Copyright (C) 2009-2011 Raphael Kubo da Costa <kubito@gmail.com>
+ * Copyright (C) 2009-2011 Raphael Kubo da Costa <rakuco@FreeBSD.org>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -86,7 +86,7 @@ bool CliInterface::list()
     QStringList args = m_param.value(ListArgs).toStringList();
     substituteListVariables(args);
 
-    if (!runProcess(m_param.value(ListProgram).toString(), args)) {
+    if (!runProcess(m_param.value(ListProgram).toStringList(), args)) {
         failOperation();
         return false;
     }
@@ -150,7 +150,7 @@ bool CliInterface::copyFiles(const QList<QVariant> & files, const QString & dest
                 kDebug() << "Password hint enabled, querying user";
 
                 Kerfuffle::PasswordNeededQuery query(filename());
-                userQuery(&query);
+                emit userQuery(&query);
                 query.waitForResponse();
 
                 if (query.responseCancelled()) {
@@ -225,7 +225,7 @@ bool CliInterface::copyFiles(const QList<QVariant> & files, const QString & dest
     kDebug() << "Setting current dir to " << destinationDirectory;
     QDir::setCurrent(destinationDirectory);
 
-    if (!runProcess(m_param.value(ExtractProgram).toString(), args)) {
+    if (!runProcess(m_param.value(ExtractProgram).toStringList(), args)) {
         failOperation();
         return false;
     }
@@ -275,7 +275,7 @@ bool CliInterface::addFiles(const QStringList & files, const CompressionOptions&
         }
     }
 
-    if (!runProcess(m_param.value(AddProgram).toString(), args)) {
+    if (!runProcess(m_param.value(AddProgram).toStringList(), args)) {
         failOperation();
         return false;
     }
@@ -310,7 +310,7 @@ bool CliInterface::deleteFiles(const QList<QVariant> & files)
 
     m_removedFiles = files;
 
-    if (!runProcess(m_param.value(DeleteProgram).toString(), args)) {
+    if (!runProcess(m_param.value(DeleteProgram).toStringList(), args)) {
         failOperation();
         return false;
     }
@@ -318,11 +318,18 @@ bool CliInterface::deleteFiles(const QList<QVariant> & files)
     return true;
 }
 
-bool CliInterface::runProcess(const QString& programName, const QStringList& arguments)
+bool CliInterface::runProcess(const QStringList& programNames, const QStringList& arguments)
 {
-    const QString programPath(KStandardDirs::findExe(programName));
+    QString programPath;
+    for (int i = 0; i < programNames.count(); i++) {
+        programPath = KStandardDirs::findExe(programNames.at(i));
+        if (!programPath.isEmpty())
+            break;
+    }
     if (programPath.isEmpty()) {
-        error(i18nc("@info", "Failed to locate program <filename>%1</filename> in PATH.", programName));
+        const QString names = programNames.join(QLatin1String(", "));
+        emit error(i18ncp("@info", "Failed to locate program <filename>%2</filename> on disk.",
+                                   "Failed to locate programs <filename>%2</filename> on disk.", programNames.count(), names));
         return false;
     }
 
@@ -334,9 +341,9 @@ bool CliInterface::runProcess(const QString& programName, const QStringList& arg
     }
 
 #ifdef Q_OS_WIN
-    m_process = new KProcess();
+    m_process = new KProcess;
 #else
-    m_process = new KPtyProcess();
+    m_process = new KPtyProcess;
     m_process->setPtyChannels(KPtyProcess::StdinChannel);
 #endif
 
@@ -379,14 +386,14 @@ void CliInterface::processFinished(int exitCode, QProcess::ExitStatus exitStatus
 
     if (m_operationMode == Delete) {
         foreach(const QVariant& v, m_removedFiles) {
-            entryRemoved(v.toString());
+            emit entryRemoved(v.toString());
         }
     }
 
     //handle all the remaining data in the process
     readStdout(true);
 
-    progress(1.0);
+    emit progress(1.0);
 
     if (m_operationMode == Add) {
         list();
@@ -394,7 +401,7 @@ void CliInterface::processFinished(int exitCode, QProcess::ExitStatus exitStatus
     }
 
     //and we're finished
-    finished(true);
+    emit finished(true);
 }
 
 void CliInterface::failOperation()
@@ -403,7 +410,7 @@ void CliInterface::failOperation()
 
     doKill();
 
-    finished(false);
+    emit finished(false);
 }
 
 void CliInterface::readStdout(bool handleAll)
@@ -437,6 +444,11 @@ void CliInterface::readStdout(bool handleAll)
     //queries (such as file exists, wrong password) on a new line, but
     //freeze waiting for input. So we check for errors on the last line in
     //all cases.
+    // TODO: QLatin1String() might not be the best choice here.
+    //       The call to handleLine() at the end of the method uses
+    //       QString::fromLocal8Bit(), for example.
+    // TODO: The same check methods are called in handleLine(), this
+    //       is suboptimal.
     bool foundErrorMessage =
         (checkForErrorMessage(QLatin1String( lines.last() ), WrongPasswordPatterns) ||
          checkForErrorMessage(QLatin1String( lines.last() ), ExtractionFailedPatterns) ||
@@ -474,12 +486,14 @@ void CliInterface::readStdout(bool handleAll)
 
 void CliInterface::handleLine(const QString& line)
 {
+    // TODO: This should be implemented by each plugin; the way progress is
+    //       shown by each CLI application is subject to a lot of variation.
     if ((m_operationMode == Copy || m_operationMode == Add) && m_param.contains(CaptureProgress) && m_param.value(CaptureProgress).toBool()) {
         //read the percentage
         int pos = line.indexOf(QLatin1Char( '%' ));
         if (pos != -1 && pos > 1) {
             int percentage = line.mid(pos - 2, 2).toInt();
-            progress(float(percentage) / 100);
+            emit progress(float(percentage) / 100);
             return;
         }
     }
@@ -489,7 +503,7 @@ void CliInterface::handleLine(const QString& line)
             kDebug() << "Found a password prompt";
 
             Kerfuffle::PasswordNeededQuery query(filename());
-            userQuery(&query);
+            emit userQuery(&query);
             query.waitForResponse();
 
             if (query.responseCancelled()) {
@@ -507,14 +521,14 @@ void CliInterface::handleLine(const QString& line)
 
         if (checkForErrorMessage(line, WrongPasswordPatterns)) {
             kDebug() << "Wrong password!";
-            error(i18n("Incorrect password."));
+            emit error(i18n("Incorrect password."));
             failOperation();
             return;
         }
 
         if (checkForErrorMessage(line, ExtractionFailedPatterns)) {
             kDebug() << "Error in extraction!!";
-            error(i18n("Extraction failed because of an unexpected error."));
+            emit error(i18n("Extraction failed because of an unexpected error."));
             failOperation();
             return;
         }
@@ -529,7 +543,7 @@ void CliInterface::handleLine(const QString& line)
             kDebug() << "Found a password prompt";
 
             Kerfuffle::PasswordNeededQuery query(filename());
-            userQuery(&query);
+            emit userQuery(&query);
             query.waitForResponse();
 
             if (query.responseCancelled()) {
@@ -547,14 +561,14 @@ void CliInterface::handleLine(const QString& line)
 
         if (checkForErrorMessage(line, WrongPasswordPatterns)) {
             kDebug() << "Wrong password!";
-            error(i18n("Incorrect password."));
+            emit error(i18n("Incorrect password."));
             failOperation();
             return;
         }
 
         if (checkForErrorMessage(line, ExtractionFailedPatterns)) {
             kDebug() << "Error in extraction!!";
-            error(i18n("Extraction failed because of an unexpected error."));
+            emit error(i18n("Extraction failed because of an unexpected error."));
             failOperation();
             return;
         }
@@ -609,7 +623,7 @@ bool CliInterface::handleFileExistsMessage(const QString& line)
 
     Kerfuffle::OverwriteQuery query(QDir::current().path() + QLatin1Char( '/' ) + filename);
     query.setNoRenameMode(true);
-    userQuery(&query);
+    emit userQuery(&query);
     kDebug() << "Waiting response";
     query.waitForResponse();
 
@@ -644,10 +658,11 @@ bool CliInterface::handleFileExistsMessage(const QString& line)
 
 bool CliInterface::checkForErrorMessage(const QString& line, int parameterIndex)
 {
+    static QHash<int, QList<QRegExp> > patternCache;
     QList<QRegExp> patterns;
 
-    if (m_patternCache.contains(parameterIndex)) {
-        patterns = m_patternCache.value(parameterIndex);
+    if (patternCache.contains(parameterIndex)) {
+        patterns = patternCache.value(parameterIndex);
     } else {
         if (!m_param.contains(parameterIndex)) {
             return false;
@@ -656,7 +671,7 @@ bool CliInterface::checkForErrorMessage(const QString& line, int parameterIndex)
         foreach(const QString& rawPattern, m_param.value(parameterIndex).toStringList()) {
             patterns << QRegExp(rawPattern);
         }
-        m_patternCache[parameterIndex] = patterns;
+        patternCache[parameterIndex] = patterns;
     }
 
     foreach(const QRegExp& pattern, patterns) {
