@@ -274,16 +274,13 @@ int ArchiveNode::row() const
 }
 
 ArchiveModel::ArchiveModel(QObject *parent)
-        : QAbstractItemModel(parent), m_archive(0),
+        : QAbstractItemModel(parent),
         m_rootNode(new ArchiveDirNode(0, ArchiveEntry()))
 {
 }
 
 ArchiveModel::~ArchiveModel()
 {
-    delete m_archive;
-    m_archive = 0;
-
     delete m_rootNode;
     m_rootNode = 0;
 }
@@ -867,18 +864,17 @@ void ArchiveModel::insertNode(ArchiveNode *node, InsertBehaviour behaviour)
 
 Kerfuffle::Archive* ArchiveModel::archive() const
 {
-    return m_archive;
+    return m_archive.data();
 }
 
 KJob* ArchiveModel::setArchive(Kerfuffle::Archive *archive)
 {
-    delete m_archive;
-    m_archive = archive;
+    m_archive.reset(archive);
+
     m_rootNode->clear();
     s_previousMatch = 0;
     s_previousPieces->clear();
 
-
     Kerfuffle::ListJob *job = NULL;
 
     m_newArchiveEntries.clear();
